@@ -13,12 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from urllib import parse as urlparse
+
 from oslo_db.exception import DBDuplicateEntry
 from oslo_log import log as logging
 from oslo_utils import strutils
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-import six.moves.urllib.parse as urlparse
 
 from nova.db import api as db
 from nova import exception
@@ -34,7 +35,10 @@ LOG = logging.getLogger(__name__)
 @base.NovaObjectRegistry.register
 class ConsoleAuthToken(base.NovaTimestampObject, base.NovaObject):
     # Version 1.0: Initial version
-    VERSION = '1.0'
+    # Version 1.1: Add clean_expired_console_auths method.
+    #              The clean_expired_console_auths_for_host method
+    #              was deprecated.
+    VERSION = '1.1'
 
     fields = {
         'id': fields.IntegerField(),
@@ -85,24 +89,6 @@ class ConsoleAuthToken(base.NovaTimestampObject, base.NovaObject):
         obj.obj_reset_changes()
         return obj
 
-    def to_dict(self):
-        """Convert to a dict representation."""
-        # NOTE(PaulMurray) For compatibility while there is code that
-        # expects the dict representation returned by consoleauth.
-        # TODO(PaulMurray) Remove this function when the code no
-        # longer expects the consoleauth dict representation
-        connect_info = {}
-        connect_info['token'] = self.token,
-        connect_info['instance_uuid'] = self.instance_uuid
-        connect_info['console_type'] = self.console_type
-        connect_info['host'] = self.host
-        connect_info['port'] = self.port
-        if 'internal_access_path' in self:
-            connect_info['internal_access_path'] = self.internal_access_path
-        if 'access_url_base' in self:
-            connect_info['access_url'] = self.access_url
-        return connect_info
-
     @base.remotable
     def authorize(self, ttl):
         """Authorise the console token and store in the database.
@@ -184,6 +170,19 @@ class ConsoleAuthToken(base.NovaTimestampObject, base.NovaObject):
         """
         db.console_auth_token_destroy_all_by_instance(context, instance_uuid)
 
+    @base.remotable_classmethod
+    def clean_expired_console_auths(cls, context):
+        """Remove all expired console authorizations.
+
+        :param context: the context
+
+        All expired authorizations will be removed.
+        Tokens that have not expired will remain.
+        """
+        db.console_auth_token_destroy_expired(context)
+
+    # TODO(takashin): This method was deprecated and will be removed
+    # in a next major version bump.
     @base.remotable_classmethod
     def clean_expired_console_auths_for_host(cls, context, host):
         """Remove all expired console authorizations for the host.
