@@ -18,7 +18,6 @@ from nova import test
 from nova.tests import fixtures as nova_fixtures
 from nova.tests.functional.api import client
 from nova.tests.unit import cast_as_call
-import nova.tests.unit.image.fake
 from nova.tests.unit import policy_fixture
 
 
@@ -50,13 +49,12 @@ class TestDeleteFromCell0CheckQuota(test.TestCase):
         super(TestDeleteFromCell0CheckQuota, self).setUp()
         self.useFixture(policy_fixture.RealPolicyFixture())
         self.useFixture(nova_fixtures.NeutronFixture(self))
+        self.useFixture(nova_fixtures.GlanceFixture(self))
+
         api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
             api_version='v2.1'))
         self.api = api_fixture.api
 
-        # the image fake backend needed for image discovery
-        nova.tests.unit.image.fake.stub_out_image_service(self)
-
         self.start_service('conductor')
         self.start_service('scheduler')
 
