@@ -22,7 +22,6 @@ from nova.tests.functional import fixtures as func_fixtures
 from nova.tests.functional import integrated_helpers
 from nova.tests.unit import fake_network
 from nova.tests.unit import fake_notifier
-import nova.tests.unit.image.fake
 from nova.tests.unit import policy_fixture
 
 
@@ -47,6 +46,7 @@ class FailedEvacuateStateTests(test.TestCase,
 
         self.useFixture(policy_fixture.RealPolicyFixture())
         self.useFixture(nova_fixtures.NeutronFixture(self))
+        self.useFixture(nova_fixtures.GlanceFixture(self))
         self.useFixture(func_fixtures.PlacementFixture())
 
         api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
@@ -55,13 +55,9 @@ class FailedEvacuateStateTests(test.TestCase,
         self.api = api_fixture.admin_api
         self.api.microversion = self.microversion
 
-        nova.tests.unit.image.fake.stub_out_image_service(self)
-
         self.start_service('conductor')
         self.start_service('scheduler')
 
-        self.addCleanup(nova.tests.unit.image.fake.FakeImageService_reset)
-
         self.hostname = 'host1'
         self.compute1 = self.start_service('compute', host=self.hostname)
         fake_network.set_stub_network_methods(self)
@@ -100,14 +96,11 @@ class FailedEvacuateStateTests(test.TestCase,
         fake_notifier.reset()
 
         # Initiate evacuation
-        post = {'evacuate': {}}
-        self.api.post_server_action(server['id'], post)
-
+        self._evacuate_server(
+            server, expected_state='ERROR', expected_host=self.hostname,
+            expected_migration_status='error')
         self._wait_for_notification_event_type('compute_task.rebuild_server')
 
-        server = self._wait_for_state_change(server, 'ERROR')
-        self.assertEqual(self.hostname, server['OS-EXT-SRV-ATTR:host'])
-
         # Check migrations
         migrations = self.api.get_migrations()
         self.assertEqual(1, len(migrations))
