@@ -19,8 +19,6 @@ import sys
 import mock
 from oslo_service import fixture as service_fixture
 from oslo_utils import encodeutils
-import six
-import testtools
 
 from nova import context
 from nova import exception
@@ -61,12 +59,16 @@ class GuestTestCase(test.NoDBTestCase):
         libvirt_guest.Guest.create("xml", self.host)
         mock_define.assert_called_once_with("xml")
 
+    @mock.patch.object(libvirt_guest.LOG, 'error')
     @mock.patch.object(fakelibvirt.Connection, 'defineXML')
-    def test_create_exception(self, mock_define):
+    def test_create_exception(self, mock_define, mock_log):
+        fake_xml = '<test>this is a test</test>'
         mock_define.side_effect = test.TestingException
         self.assertRaises(test.TestingException,
                           libvirt_guest.Guest.create,
-                          "foo", self.host)
+                          fake_xml, self.host)
+        # ensure the XML is logged
+        self.assertIn(fake_xml, str(mock_log.call_args[0]))
 
     def test_launch(self):
         self.guest.launch()
@@ -77,18 +79,22 @@ class GuestTestCase(test.NoDBTestCase):
         self.domain.createWithFlags.assert_called_once_with(
             fakelibvirt.VIR_DOMAIN_START_PAUSED)
 
+    @mock.patch.object(libvirt_guest.LOG, 'exception')
+    @mock.patch.object(encodeutils, 'safe_decode')
+    def test_launch_exception(self, mock_safe_decode, mock_log):
+        fake_xml = '<test>this is a test</test>'
+        self.domain.createWithFlags.side_effect = test.TestingException
+        mock_safe_decode.return_value = fake_xml
+        self.assertRaises(test.TestingException, self.guest.launch)
+        self.assertEqual(1, mock_safe_decode.called)
+        # ensure the XML is logged
+        self.assertIn(fake_xml, str(mock_log.call_args[0]))
+
     def test_shutdown(self):
         self.domain.shutdown = mock.MagicMock()
         self.guest.shutdown()
         self.domain.shutdown.assert_called_once_with()
 
-    @mock.patch.object(encodeutils, 'safe_decode')
-    def test_launch_exception(self, mock_safe_decode):
-        self.domain.createWithFlags.side_effect = test.TestingException
-        mock_safe_decode.return_value = "</xml>"
-        self.assertRaises(test.TestingException, self.guest.launch)
-        self.assertEqual(1, mock_safe_decode.called)
-
     def test_get_interfaces(self):
         self.domain.XMLDesc.return_value = """<domain>
   <devices>
@@ -273,7 +279,7 @@ class GuestTestCase(test.NoDBTestCase):
         ex = self.assertRaises(
             exception.DeviceNotFound, self.guest.detach_device_with_retry,
             get_config, "/dev/vdb", live=True)
-        self.assertIn("/dev/vdb", six.text_type(ex))
+        self.assertIn("/dev/vdb", str(ex))
 
     def test_detach_device_with_retry_device_not_found_alt_name(self):
         """Tests to make sure we use the alternative name in errors."""
@@ -283,7 +289,7 @@ class GuestTestCase(test.NoDBTestCase):
             exception.DeviceNotFound, self.guest.detach_device_with_retry,
             get_config, mock.sentinel.device, live=True,
             alternative_device_name='foo')
-        self.assertIn('foo', six.text_type(ex))
+        self.assertIn('foo', str(ex))
 
     @mock.patch.object(libvirt_guest.Guest, "detach_device")
     def _test_detach_device_with_retry_second_detach_failure(
@@ -305,8 +311,7 @@ class GuestTestCase(test.NoDBTestCase):
         mock_detach.side_effect = [None, fake_exc]
         retry_detach = self.guest.detach_device_with_retry(
             get_config, fake_device, live=True,
-            inc_sleep_time=.01, max_retry_count=3,
-            supports_device_missing_error_code=supports_device_missing)
+            inc_sleep_time=.01, max_retry_count=3)
         # Some time later, we can do the wait/retry to ensure detach
         self.assertRaises(exception.DeviceNotFound, retry_detach)
         # Check that the save_and_reraise_exception context manager didn't log
@@ -315,20 +320,6 @@ class GuestTestCase(test.NoDBTestCase):
         self.assertNotIn('Original exception being dropped',
                          self.stdlog.logger.output)
 
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_second_detach_operation_failed(self):
-        self._test_detach_device_with_retry_second_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_OPERATION_FAILED,
-            error_message="operation failed: disk vdb not found",
-            supports_device_missing=False)
-
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_second_detach_internal_error(self):
-        self._test_detach_device_with_retry_second_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_INTERNAL_ERROR,
-            error_message="operation failed: disk vdb not found",
-            supports_device_missing=False)
-
     def test_detach_device_with_retry_second_detach_device_missing(self):
         self._test_detach_device_with_retry_second_detach_failure(
             error_code=fakelibvirt.VIR_ERR_DEVICE_MISSING,
@@ -368,8 +359,7 @@ class GuestTestCase(test.NoDBTestCase):
         # succeeds afterward
         self.domain.detachDeviceFlags.side_effect = [fake_exc, None]
         retry_detach = self.guest.detach_device_with_retry(get_config,
-            fake_device, live=True, inc_sleep_time=.01, max_retry_count=3,
-            supports_device_missing_error_code=supports_device_missing)
+            fake_device, live=True, inc_sleep_time=.01, max_retry_count=3)
         # We should have tried to detach from the persistent domain
         self.domain.detachDeviceFlags.assert_called_once_with(
             "</xml>", flags=(fakelibvirt.VIR_DOMAIN_AFFECT_CONFIG |
@@ -381,33 +371,54 @@ class GuestTestCase(test.NoDBTestCase):
         self.domain.detachDeviceFlags.assert_called_once_with(
             "</xml>", flags=fakelibvirt.VIR_DOMAIN_AFFECT_LIVE)
 
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_first_detach_operation_failed(self):
-        self._test_detach_device_with_retry_first_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_OPERATION_FAILED,
-            error_message="operation failed: disk vdb not found",
-            supports_device_missing=False)
-
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_first_detach_internal_error(self):
-        self._test_detach_device_with_retry_first_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_INTERNAL_ERROR,
-            error_message="operation failed: disk vdb not found",
-            supports_device_missing=False)
-
-    # TODO(lyarwood): Remove this test once MIN_LIBVIRT_VERSION is >= 4.1.0
-    def test_detach_device_with_retry_first_detach_invalid_arg(self):
-        self._test_detach_device_with_retry_first_detach_failure(
-            error_code=fakelibvirt.VIR_ERR_INVALID_ARG,
-            error_message="invalid argument: no target device vdb",
-            supports_device_missing=False)
-
     def test_detach_device_with_retry_first_detach_device_missing(self):
         self._test_detach_device_with_retry_first_detach_failure(
             error_code=fakelibvirt.VIR_ERR_DEVICE_MISSING,
             error_message="device not found: disk vdb not found",
             supports_device_missing=True)
 
+    def test_detach_device_with_already_in_process_of_unplug_error(self):
+        # Assert that DeviceNotFound is raised when encountering
+        # https://bugzilla.redhat.com/show_bug.cgi?id=1878659
+        # This is raised as QEMU returns a VIR_ERR_INTERNAL_ERROR when
+        # a request to device_del is made while another is about to complete.
+
+        self.domain.isPersistent.return_value = True
+        conf = mock.Mock(spec=vconfig.LibvirtConfigGuestDevice)
+        conf.to_xml.return_value = "</xml>"
+
+        existing_unplug_exc = fakelibvirt.make_libvirtError(
+            fakelibvirt.libvirtError, "",
+            error_message='device vdb is already in the process of unplug',
+            error_code=fakelibvirt.VIR_ERR_INTERNAL_ERROR,
+            error_domain=fakelibvirt.VIR_FROM_DOMAIN
+        )
+        device_missing_exc = fakelibvirt.make_libvirtError(
+            fakelibvirt.libvirtError, "",
+            error_message='device not found: disk vdb not found',
+            error_code=fakelibvirt.VIR_ERR_DEVICE_MISSING,
+            error_domain=fakelibvirt.VIR_FROM_DOMAIN
+        )
+
+        # Raise VIR_ERR_INTERNAL_ERROR on the second call before raising
+        # VIR_ERR_DEVICE_MISSING to mock the first call successfully detaching
+        # the device asynchronously.
+        self.domain.detachDeviceFlags.side_effect = [
+            None,
+            existing_unplug_exc,
+            device_missing_exc
+        ]
+
+        retry_detach = self.guest.detach_device_with_retry(
+            mock.Mock(return_value=conf),
+            'vdb',
+            live=True,
+            inc_sleep_time=.01
+        )
+
+        # Assert that we raise exception.DeviceNotFound
+        self.assertRaises(exception.DeviceNotFound, retry_detach)
+
     def test_get_xml_desc(self):
         self.guest.get_xml_desc()
         self.domain.XMLDesc.assert_called_once_with(flags=0)
@@ -571,6 +582,49 @@ class GuestTestCase(test.NoDBTestCase):
         self.assertIsNotNone(
             self.guest.get_interface_by_cfg(cfg))
         self.assertIsNone(self.guest.get_interface_by_cfg(None))
+        self.domain.XMLDesc.assert_has_calls([mock.call(0)] * 6)
+
+        # now check if the persistent config can be queried too
+        self.domain.XMLDesc.reset_mock()
+        devs = self.guest.get_all_devices(
+            devtype=None, from_persistent_config=True)
+        self.domain.XMLDesc.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_XML_INACTIVE)
+
+    def test_get_interface_by_cfg_persistent_domain(self):
+        self.domain.XMLDesc.return_value = """<domain>
+  <devices>
+    <interface type="bridge">
+      <mac address="fa:16:3e:f9:af:ae"/>
+      <model type="virtio"/>
+      <driver name="qemu"/>
+      <source bridge="qbr84008d03-11"/>
+      <target dev="tap84008d03-11"/>
+    </interface>
+  </devices>
+</domain>"""
+        cfg = vconfig.LibvirtConfigGuestInterface()
+        cfg.parse_str("""
+            <interface type="bridge">
+              <mac address="fa:16:3e:f9:af:ae"/>
+              <model type="virtio"/>
+              <driver name="qemu"/>
+              <source bridge="qbr84008d03-11"/>
+              <target dev="tap84008d03-11"/>
+              </interface>""")
+        self.assertIsNotNone(
+            self.guest.get_interface_by_cfg(
+                cfg, from_persistent_config=True))
+        self.assertIsNone(
+            self.guest.get_interface_by_cfg(
+                vconfig.LibvirtConfigGuestInterface(),
+                from_persistent_config=True))
+        self.domain.XMLDesc.assert_has_calls(
+            [
+                mock.call(fakelibvirt.VIR_DOMAIN_XML_INACTIVE),
+                mock.call(fakelibvirt.VIR_DOMAIN_XML_INACTIVE),
+            ]
+        )
 
     def test_get_interface_by_cfg_vhostuser(self):
         self.domain.XMLDesc.return_value = """<domain>
@@ -600,6 +654,44 @@ class GuestTestCase(test.NoDBTestCase):
             self.guest.get_interface_by_cfg(cfg))
         self.assertIsNone(self.guest.get_interface_by_cfg(None))
 
+    def test_get_interface_by_cfg_hostdev_pci(self):
+        self.domain.XMLDesc.return_value = """<domain>
+            <devices>
+                <hostdev mode='subsystem' type='pci' managed='yes'>
+                <driver name='vfio'/>
+                <source>
+                <address domain='0x0000' bus='0x81' slot='0x00'
+                    function='0x1'/>
+                </source>
+                <alias name='hostdev0'/>
+                <address type='pci' domain='0x0000' bus='0x00' slot='0x04'
+                    function='0x0'/>
+                </hostdev>
+            </devices>
+        </domain>"""
+        cfg = vconfig.LibvirtConfigGuestHostdevPCI()
+        cfg.parse_str("""
+            <hostdev mode='subsystem' type='pci' managed='yes'>
+            <driver name='vfio'/>
+            <source>
+            <address domain='0x0000' bus='0x81' slot='0x00' function='0x1'/>
+            </source>
+            </hostdev>""")
+        self.assertIsNotNone(
+            self.guest.get_interface_by_cfg(cfg))
+
+        cfg.parse_str("""
+            <hostdev mode='subsystem' type='pci' managed='yes'>
+            <driver name='vfio'/>
+            <source>
+            <address domain='0000' bus='81' slot='00' function='1'/>
+            </source>
+            </hostdev>""")
+        self.assertIsNotNone(
+            self.guest.get_interface_by_cfg(cfg))
+
+        self.assertIsNone(self.guest.get_interface_by_cfg(None))
+
     def test_get_info(self):
         self.domain.info.return_value = (1, 2, 3, 4, 5)
         self.domain.ID.return_value = 6
@@ -698,24 +790,6 @@ class GuestTestCase(test.NoDBTestCase):
                                            'persistent_xml': '</xml>',
                                            'bandwidth': 2})
 
-    @testtools.skipIf(not six.PY2, 'libvirt python3 bindings accept unicode')
-    def test_migrate_v3_unicode(self):
-        dest_xml_template = "<domain type='qemu'><name>%s</name></domain>"
-        name = u'\u00CD\u00F1st\u00E1\u00F1c\u00E9'
-        dest_xml = dest_xml_template % name
-        expect_dest_xml = dest_xml_template % encodeutils.to_utf8(name)
-        self.guest.migrate('an-uri', flags=1, migrate_uri='dest-uri',
-                           migrate_disks=[u"disk1", u"disk2"],
-                           destination_xml=dest_xml,
-                           bandwidth=2)
-        self.domain.migrateToURI3.assert_called_once_with(
-                'an-uri', flags=1, params={'migrate_uri': 'dest-uri',
-                                           'migrate_disks': ['disk1',
-                                                             'disk2'],
-                                           'destination_xml': expect_dest_xml,
-                                           'persistent_xml': expect_dest_xml,
-                                           'bandwidth': 2})
-
     def test_abort_job(self):
         self.guest.abort_job()
         self.domain.abortJob.assert_called_once_with()
@@ -724,6 +798,39 @@ class GuestTestCase(test.NoDBTestCase):
         self.guest.migrate_configure_max_downtime(1000)
         self.domain.migrateSetMaxDowntime.assert_called_once_with(1000)
 
+    def test_set_metadata(self):
+        meta = mock.Mock(spec=vconfig.LibvirtConfigGuestMetaNovaInstance)
+        meta.to_xml.return_value = "</xml>"
+        self.guest.set_metadata(meta)
+        self.domain.setMetadata.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_METADATA_ELEMENT, "</xml>", "instance",
+            vconfig.NOVA_NS, flags=0)
+
+    def test_set_metadata_persistent(self):
+        meta = mock.Mock(spec=vconfig.LibvirtConfigGuestMetaNovaInstance)
+        meta.to_xml.return_value = "</xml>"
+        self.guest.set_metadata(meta, persistent=True)
+        self.domain.setMetadata.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_METADATA_ELEMENT, "</xml>", "instance",
+            vconfig.NOVA_NS, flags=fakelibvirt.VIR_DOMAIN_AFFECT_CONFIG)
+
+    def test_set_metadata_device_live(self):
+        meta = mock.Mock(spec=vconfig.LibvirtConfigGuestMetaNovaInstance)
+        meta.to_xml.return_value = "</xml>"
+        self.guest.set_metadata(meta, live=True)
+        self.domain.setMetadata.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_METADATA_ELEMENT, "</xml>", "instance",
+            vconfig.NOVA_NS, flags=fakelibvirt.VIR_DOMAIN_AFFECT_LIVE)
+
+    def test_set_metadata_persistent_live(self):
+        meta = mock.Mock(spec=vconfig.LibvirtConfigGuestMetaNovaInstance)
+        meta.to_xml.return_value = "</xml>"
+        self.guest.set_metadata(meta, persistent=True, live=True)
+        self.domain.setMetadata.assert_called_once_with(
+            fakelibvirt.VIR_DOMAIN_METADATA_ELEMENT, "</xml>", "instance",
+            vconfig.NOVA_NS, flags=fakelibvirt.VIR_DOMAIN_AFFECT_LIVE |
+                                   fakelibvirt.VIR_DOMAIN_AFFECT_CONFIG)
+
 
 class GuestBlockTestCase(test.NoDBTestCase):
 
