@@ -502,10 +502,10 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
         self.assertEqual(0, unit_number)
         self.assertEqual(1, controller_spec.device.busNumber)
 
-    def _test_get_vnc_config_spec(self, keymap):
+    def test_get_vnc_config_spec(self):
+        self.flags(vnc_keymap='en-ie', group='vmware')
         fake_factory = fake.FakeFactory()
-        result = vm_util.get_vnc_config_spec(fake_factory,
-                                             7)
+        result = vm_util.get_vnc_config_spec(fake_factory, 7)
         expected = fake_factory.create('ns0:VirtualMachineConfigSpec')
         expected.extraConfig = []
 
@@ -520,23 +520,12 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
         expected.extraConfig.append(remote_display_vnc_port)
 
         remote_display_vnc_keymap = fake_factory.create('ns0:OptionValue')
-        remote_display_vnc_keymap.value = keymap
+        remote_display_vnc_keymap.value = 'en-ie'
         remote_display_vnc_keymap.key = 'RemoteDisplay.vnc.keyMap'
         expected.extraConfig.append(remote_display_vnc_keymap)
 
         self.assertEqual(expected, result)
 
-    def test_get_vnc_config_spec(self):
-        # TODO(stephenfin): Fold this back in and stop overridding the keymap
-        # option once we remove the '[vnc] keymap' option
-        self.flags(vnc_keymap='en-ie', group='vmware')
-        self._test_get_vnc_config_spec('en-ie')
-
-    def test_get_vnc_config_spec__legacy_keymap(self):
-        self.flags(keymap='en-uk', group='vnc')
-        self.flags(vnc_keymap='en-ie', group='vmware')
-        self._test_get_vnc_config_spec('en-uk')
-
     def _create_fake_vms(self):
         fake_vms = fake.FakeRetrieveResult()
         OptionValue = collections.namedtuple('OptionValue', ['key', 'value'])
@@ -1931,8 +1920,8 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
         session = fake.FakeSession()
         with mock.patch.object(session, '_call_method',
                                side_effect=[child_folder]):
-            parent_folder = mock.sentinel.parent_folder
-            parent_folder.value = 'parent-ref'
+            parent_folder = fake.ManagedObjectReference(value='parent-ref',
+                                                        name='Folder')
             child_name = 'child_folder'
             ret = vm_util.create_folder(session, parent_folder, child_name)
 
@@ -1949,8 +1938,8 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
         duplicate_exception = vexc.DuplicateName(details=details)
         with mock.patch.object(session, '_call_method',
                                side_effect=[duplicate_exception]):
-            parent_folder = mock.sentinel.parent_folder
-            parent_folder.value = 'parent-ref'
+            parent_folder = fake.ManagedObjectReference(value='parent-ref',
+                                                        name='Folder')
             child_name = 'child_folder'
             ret = vm_util.create_folder(session, parent_folder, child_name)
 
