@@ -155,9 +155,8 @@ class Guest(object):
             return self._domain.createWithFlags(flags)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error('Error launching a defined domain '
-                          'with XML: %s',
-                          self._encoded_xml, errors='ignore')
+                LOG.exception('Error launching a defined domain with XML: %s',
+                              self._encoded_xml, errors='ignore')
 
     def poweroff(self):
         """Stops a running guest."""
@@ -228,19 +227,22 @@ class Guest(object):
 
         return interfaces
 
-    def get_interface_by_cfg(self, cfg):
+    def get_interface_by_cfg(self, cfg, from_persistent_config=False):
         """Lookup a full LibvirtConfigGuestDevice with
         LibvirtConfigGuesDevice generated
         by nova.virt.libvirt.vif.get_config.
 
         :param cfg: config object that represents the guest interface.
         :type cfg: a subtype of LibvirtConfigGuestDevice object
+        :param from_persistent_config: query the device from the persistent
+            domain instead of the live domain configuration
         :returns: nova.virt.libvirt.config.LibvirtConfigGuestDevice instance
             if found, else None
         """
 
         if cfg:
-            interfaces = self.get_all_devices(type(cfg))
+            interfaces = self.get_all_devices(
+                type(cfg), from_persistent_config)
             for interface in interfaces:
                 # NOTE(leehom) LibvirtConfigGuest get from domain and
                 # LibvirtConfigGuest generated by
@@ -304,6 +306,28 @@ class Guest(object):
         LOG.debug("attach device xml: %s", device_xml)
         self._domain.attachDeviceFlags(device_xml, flags=flags)
 
+    def set_metadata(self, metadata, persistent=False, live=False):
+        """Set metadata to the guest.
+
+        Please note that this function completely replaces the existing
+        metadata. The scope of the replacement is limited to the Nova-specific
+        XML Namespace.
+
+        :param metadata: A LibvirtConfigGuestMetaNovaInstance
+        :param persistent: A bool to indicate whether the change is
+                           persistent or not
+        :param live: A bool to indicate whether it affect the guest
+                     in running state
+        """
+        flags = persistent and libvirt.VIR_DOMAIN_AFFECT_CONFIG or 0
+        flags |= live and libvirt.VIR_DOMAIN_AFFECT_LIVE or 0
+
+        metadata_xml = metadata.to_xml()
+        LOG.debug("set metadata xml: %s", metadata_xml)
+        self._domain.setMetadata(libvirt.VIR_DOMAIN_METADATA_ELEMENT,
+                                 metadata_xml, "instance",
+                                 vconfig.NOVA_NS, flags=flags)
+
     def get_config(self):
         """Returns the config instance for a guest
 
@@ -313,13 +337,16 @@ class Guest(object):
         config.parse_str(self._domain.XMLDesc(0))
         return config
 
-    def get_disk(self, device):
+    def get_disk(self, device, from_persistent_config=False):
         """Returns the disk mounted at device
 
         :returns LivirtConfigGuestDisk: mounted at device or None
         """
+        flags = 0
+        if from_persistent_config:
+            flags |= libvirt.VIR_DOMAIN_XML_INACTIVE
         try:
-            doc = etree.fromstring(self._domain.XMLDesc(0))
+            doc = etree.fromstring(self._domain.XMLDesc(flags))
         except Exception:
             return None
 
@@ -344,18 +371,24 @@ class Guest(object):
 
         return self.get_all_devices(vconfig.LibvirtConfigGuestDisk)
 
-    def get_all_devices(self, devtype=None):
+    def get_all_devices(self, devtype=None, from_persistent_config=False):
         """Returns all devices for a guest
 
         :param devtype: a LibvirtConfigGuestDevice subclass class
-
+        :param from_persistent_config: query the device from the persistent
+            domain (i.e. inactive XML configuration that'll be used on next
+            start of the domain) instead of the live domain configuration
         :returns: a list of LibvirtConfigGuestDevice instances
         """
 
+        flags = 0
+        if from_persistent_config:
+            flags |= libvirt.VIR_DOMAIN_XML_INACTIVE
+
         try:
             config = vconfig.LibvirtConfigGuest()
             config.parse_str(
-                self._domain.XMLDesc(0))
+                self._domain.XMLDesc(flags))
         except Exception:
             return []
 
@@ -369,8 +402,7 @@ class Guest(object):
     def detach_device_with_retry(self, get_device_conf_func, device, live,
                                  max_retry_count=7, inc_sleep_time=10,
                                  max_sleep_time=60,
-                                 alternative_device_name=None,
-                                 supports_device_missing_error_code=False):
+                                 alternative_device_name=None):
         """Detaches a device from the guest. After the initial detach request,
         a function is returned which can be used to ensure the device is
         successfully removed from the guest domain (retrying the removal as
@@ -391,16 +423,8 @@ class Guest(object):
                                max_sleep_time will be used as the sleep time.
         :param alternative_device_name: This is an alternative identifier for
             the device if device is not an ID, used solely for error messages.
-        :param supports_device_missing_error_code: does the installed version
-                                                   of libvirt provide the
-                                                   VIR_ERR_DEVICE_MISSING error
-                                                   code.
         """
         alternative_device_name = alternative_device_name or device
-        unplug_libvirt_error_codes = set([
-            libvirt.VIR_ERR_OPERATION_FAILED,
-            libvirt.VIR_ERR_INTERNAL_ERROR
-        ])
 
         def _try_detach_device(conf, persistent=False, live=False):
             # Raise DeviceNotFound if the device isn't found during detach
@@ -410,34 +434,27 @@ class Guest(object):
                     LOG.debug('Successfully detached device %s from guest. '
                               'Persistent? %s. Live? %s',
                               device, persistent, live)
-
             except libvirt.libvirtError as ex:
                 with excutils.save_and_reraise_exception(reraise=False) as ctx:
-                    errcode = ex.get_error_code()
-                    # TODO(lyarwood): Remove libvirt.VIR_ERR_OPERATION_FAILED
-                    # and libvirt.VIR_ERR_INTERNAL_ERROR once
-                    # MIN_LIBVIRT_VERSION is >= 4.1.0
-                    if supports_device_missing_error_code:
-                        unplug_libvirt_error_codes.add(
-                            libvirt.VIR_ERR_DEVICE_MISSING)
-                    if errcode in unplug_libvirt_error_codes:
-                        # TODO(lyarwood): Remove the following error message
-                        # check once we only care about VIR_ERR_DEVICE_MISSING
-                        errmsg = ex.get_error_message()
-                        if 'not found' in errmsg:
-                            # This will be raised if the live domain
-                            # detach fails because the device is not found
-                            raise exception.DeviceNotFound(
-                                device=alternative_device_name)
-                    # TODO(lyarwood): Remove libvirt.VIR_ERR_INVALID_ARG once
-                    # MIN_LIBVIRT_VERSION is >= 4.1.0
-                    elif errcode == libvirt.VIR_ERR_INVALID_ARG:
-                        errmsg = ex.get_error_message()
-                        if 'no target device' in errmsg:
-                            # This will be raised if the persistent domain
-                            # detach fails because the device is not found
-                            raise exception.DeviceNotFound(
-                                device=alternative_device_name)
+                    code = ex.get_error_code()
+                    msg = ex.get_error_message()
+                    if code == libvirt.VIR_ERR_DEVICE_MISSING:
+                        raise exception.DeviceNotFound(
+                            device=alternative_device_name)
+                    # NOTE(lyarwood): https://bugzilla.redhat.com/1878659
+                    # Ignore this known QEMU bug for the time being allowing
+                    # our retry logic to fire again and hopefully see that
+                    # the device has been removed asynchronously by QEMU
+                    # in the meantime when the next call to detach raises
+                    # VIR_ERR_DEVICE_MISSING.
+                    if (code == libvirt.VIR_ERR_INTERNAL_ERROR and
+                        msg and 'already in the process of unplug' in msg
+                    ):
+                        LOG.debug('Ignoring QEMU rejecting our request to '
+                                  'detach as it is caused by a previous '
+                                  'request still being in progress.')
+                        return
+
                     # Re-raise the original exception if we're not raising
                     # DeviceNotFound instead. This will avoid logging of a
                     # "Original exception being dropped" traceback.
